/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.tapestry;

import java.util.Locale;
import java.util.regex.Pattern;

import org.apache.tapestry5.services.ComponentClassResolver;
import org.apache.tapestry5.services.Request;
import org.apache.tapestry5.urlrewriter.SimpleRequestWrapper;
import org.apache.tapestry5.urlrewriter.URLRewriteContext;

/**
 * PageMappingRule is a URLRewriterRule which strips the Page suffix off from
 * pagenames in URL generation.
 * 
 * @author tiwe
 * @version $Id$
 * 
 */
@Deprecated
public class PageMappingRule extends AbstractRewriterRule {

    private static final Pattern replacePattern = Pattern.compile("page$");

    private final ComponentClassResolver resolver;

    public PageMappingRule(ComponentClassResolver resolver) {
        this.resolver = resolver;
    }

    @Override
    protected Request decodeRequest(Request request, URLRewriteContext context) {
        String path = request.getPath();
        if (path.equals("/favicon.ico") || path.startsWith("/assets")
                || path.contains(".") || path.contains(":")) {
            return request;
        }

        // strip off the leading /
        String originalPageName = request.getPath().substring(1);
        String newPageName;

        while ((newPageName = toLogicalPageName(originalPageName)) == null) {
            int index = originalPageName.lastIndexOf('/');
            if (index == -1) {
                originalPageName = "";
                newPageName = toLogicalPageName(originalPageName);
                break;
            } else {
                originalPageName = originalPageName.substring(0, index);
            }
        }

        if (newPageName == null) {
            throw new IllegalArgumentException("No valid page name in path: "
                    + request.getPath());
        }

        // the two being equal implies that we didn't actually do any rewriting
        if (newPageName.equals(originalPageName)) {
            return request;
        } else {
            String newPath = request.getPath().replaceFirst(originalPageName,
                    newPageName);
            // System.err.println(String.format("decode '%s' > '%s'",
            // request.getPath(), newPath));
            return new SimpleRequestWrapper(request, newPath);
        }

    }

    @Override
    protected Request encodeRequest(Request request, URLRewriteContext context) {
        String path = request.getPath();
        if (!path.contains(".") && !path.contains(":")) {
            String pageName = (context.getPageParameters() == null ? context
                    .getComponentEventParameters().getActivePageName()
                    : context.getPageParameters().getLogicalPageName());

            // so, map the page.
            String revised = toURLPageName(pageName);
            pageName = pageName.toLowerCase(Locale.ENGLISH);
            if (pageName.equals(revised)) {
                return request;
            } else {
                String newPath = request.getPath().replaceFirst(pageName,
                        revised);
                // System.err.println(String.format("encode '%s' > '%s'",
                // request.getPath(), newPath));
                return new SimpleRequestWrapper(request, newPath);
            }

        } else {
            return request;
        }

    }

    public String toURLPageName(String pageName) {
        return replacePattern.matcher(pageName.toLowerCase()).replaceAll("");
    }

    public String toLogicalPageName(String urlPageName) {
        if (resolver.isPageName(urlPageName)) {
            return urlPageName;
        } else {
            urlPageName += "page";
            if (resolver.isPageName(urlPageName)) {
                return urlPageName;
            }
            return null;
        }

    }
}
