/*
 * Copyright (c) 2009 Mysema Ltd.
 * All rights reserved.
 * 
 */
package com.mysema.tapestry.core.components;

import java.util.List;

import org.apache.tapestry5.Block;
import org.apache.tapestry5.annotations.IncludeStylesheet;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.services.ComponentEventLinkEncoder;
import org.apache.tapestry5.services.PageRenderRequestParameters;
import org.apache.tapestry5.services.Request;

import com.mysema.query.paging.ListSource;
import com.mysema.tapestry.core.Context;

/**
 * PagedList provides
 * 
 * @author tiwe
 * @version $Id$
 */
@SuppressWarnings("unused")
@IncludeStylesheet( { "PagedList.css" })
public class PagedList {

    @Parameter
    @Property
    private boolean bottomPager = true; // NOSONAR

    @Property
    private Context context;

    @Property
    private int currentPage = 1;

    @Inject
    private ComponentEventLinkEncoder eventLinkEncoder;

    @Property
    private int maxPage;

    @Property
    private String pageName;

    @Parameter
    private int pageSize;

    @Inject
    @Property
    private Block pagingBlock;

    @Inject
    private Request request;

    @Parameter(required = true, principal = true, autoconnect = true)
    @Property
    private ListSource<?> source;

    @Parameter
    @Property
    private boolean topPager = false; // NOSONAR

    @Parameter(principal = true)
    @Property
    private Object value;

    @Property
    private List<?> view;

    public boolean isPaged() {
        return maxPage > 1;
    }

    void setupRender() {
        PageRenderRequestParameters params = eventLinkEncoder
                .decodePageRenderRequest(request);
        pageName = params.getLogicalPageName();
        context = new Context(params.getActivationContext());

        // derive page and populate context base
        if (context.size() >= 2
                && context.get(context.size() - 2).equals("page")) {
            currentPage = Integer.valueOf(context.get(context.size() - 1)
                    .toString());
        } else {
            context.add("page").add(1);
        }

        // calculate view
        int start = (currentPage - 1) * pageSize;
        int end = start + pageSize;
        maxPage = Math.max(1, (int) Math.floor(source.size()
                / ((double) pageSize)));
        view = source.getResults(start, end);
    }

}
